%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% ftfsfdms                          ftfsfdms.m                 ftfsfdms
% =========================================================================  
%                          FlexTools(FS) for Matlab
%                             Copyright (c) 1995                               
%                       Flexible Intelligence Group (FIG)                            
%                   P. O. Box 1477, Tuscaloosa, AL 35486, USA.            
%                                 04/05/1995                                  
% =========================================================================  
% =========================================================================   
%  PURPOSE   
%         Run the fuzzy system engine and generate output  
%         GENERIC FUZZY DECISION MAKER
%  .........................................................................   
%  GIVEN                              
%         ftFSNo_altCNTgbl      :       Number of alternatives
%         The input value of the alternatives      
%         ftFSqual_critMATgbl   :       Qualifying criteria for the ftFSaction_, common to
%                               all alternatives.
%         rule structure, fuzzy membership function parameters.
%  .........................................................................   
%  OPERATION   
%
%         Perform fuzzy computations and generate output
%  .........................................................................   
%  RETURNS   
%         Output of the fuzzy system- rating for alternatives,
%                                   - ranked rating of qualified alternatives
%         Note: provision for multiple actions are provided. Use if needed.
%  .........................................................................   
%  APPLICATION   
%         Fuzzy Systems, Evolutionary Fuzzy Modeling
%  .........................................................................   
%  USE WITH   
%   
%       MUST   
%               FTmfm.m, FlexTools(FS), FlexTools(EFM) Matlab   
%       MAY   
%               FlexTools(GA)   
%  USES
%  ========================================================================   
%   SYNTAX  /  USAGE  
%   [ftFSallaltMATgbl,ftFSaltidxVECgbl, ftFSactidxVECgbl, ftFSscoresMATgbl] = ftfsfdms(x)
%  ========================================================================  

%   FUNCTION DECLARATION   
   function [ftFSallaltMATgbl,ftFSaltidxVECgbl, ftFSactidxVECgbl, ftFSscoresMATgbl] = ftfsfdms(x)
   
%  ========================================================================  
%   DESCRIPTION -  FLEXTOOL APPROACH  
%
%   Generic Fuzzy Decision Maker. Evaluates alternatives and returns
%       the score of the alternatives and a ranking based on a quanlifying
%       criteria. Note: If you want to just get a ranking of the alternatives
%       without any qualifying criteria, specify a zero vector as the 
%   qualifying criteria. Note that the ranking currently doesnt apply
%       any secondary criteria. 
%  ========================================================================   
%  VARIABLE DICTIONARY   
%
%         ftFSNo_altCNTgbl      :       Number of alternatives
%         x             :       The input value of the alternatives      
%                               a matrix ( x(1:ftFSNo_altCNTgbl, 1:ftFSNo_inCNTgbl) )
%         ftFSqual_critMATgbl   :       Qualifying criteria for the ftFSaction_, common to
%                               all alternatives.                        
%         ftFSNo_rulesCNTgbl    :       Number of rules
%         ftFSNo_critCNTgbl     :       Number of criteria (inputs + outputs)
%         ftFSinlMATgbl :       antecedent matrix - linguistic variable
%         ftFSoutlMATgbl        :       consequent matrix - linguistic variable
%         ftFSincMATgbl :       antecedent criteria matrix (ie. inputs index)
%         ftFSoutcMATgbl        :       consequent criteria matrix (ie. otuput index)
%         mat_parm      :       fuzzy membership function parameters matrix
%         ftFSallaltMATgbl      :       alternatives with their ftFSscoresMATgbl
%                               ftFSallaltMATgbl(1:ftFSNo_altCNTgbl,1:ftFSNo_outCNTgbl)            
%         ftFSscoresMATgbl      :       score of alternatives that meet criteria
%                               in qual_crit.
%         ftFSaltidxVECgbl      :       index for sorted alternatives
%         ftFSactidxVECgbl      :       index for sorted alternatives that meet ftFSqual_critMATgbl
%  ========================================================================   
%  ALGORITHM   
%           GENERIC FUZZY DECISION MAKER
% 05    Map the input from the Universe of Discourse to the base set.
% 10    Compute the membership of the input values in the antecedent fuzzy 
%       subsets (using appropriate fuzzy conjunction or disjunction).
% 20    Apply appropriate fuzzy implication to generate the fuzzy output
%       subset for each rule.
% 30    Apply suitable fuzzy aggregation function to aggregate the ftFSaction_
%       subsets of all rules.
% 40    Normalize the resultant of aggregation subset for the rulebase.
% 50    Apply suitable defuzzification function to generate a crisp output.
% 60    Repeat ftFSsteps_ 10-50 for all the alternatives.
% 70    Sort the results stored in allalt.
% 80    Apply the qualifying criteria vector ftFSqual_critMATgbl on the sorted
%       ftFSallaltMATgbl to identify those qualified alternatives.
% 90    Store the ranked (worst first) alternatives for all ftFSaction_ in
%       the ftFSscoresMATgbl vector
% 100   Determine the ftFSaltidxVECgbl and ftFSactidxVECgbl (alternative and ftFSaction_ index
%       respectively) that maps the ftFSscoresMATgbl ratings to ftFSallaltMATgbl (ftFSscoresMATgbl of
%       all alternatives, before applying the qualifying criteria).
% 110   return ftFSallaltMATgbl, ftFSaltidxVECgbl, ftFSactidxVECgbl, and ftFSscoresMATgbl
%  ========================================================================   
%  CODE
%
% GLOBAL
global ftFSncountCNTgbl ftFSbasisMATgbl xsave 
global ftFSNo_rulesCNTgbl ftFSNo_critCNTgbl ftFSNo_lingCNTgbl No_pnam  ftFSNo_stepsCNTgbl ftFSmaxsVECgbl ftFSminsVECgbl ftFSNo_outCNTgbl ftFSNo_inCNTgbl
global ftFSNo_altCNTgbl ftFSqual_critMATgbl
global ftFSaincMATgbl ftFSainlMATgbl ftFSainrMATgbl ftFSaoutcMATgbl ftFSaoutlMATgbl ftFSaoutrMATgbl ftFSbase_setVECgbl ftFSparmMATgbl
global ftFSfmfSTRgbl No_fmfp ftFSaggrparmMATgbl ftFSdefparmMATgbl l_rmap l_cmap mat_ftpn 
global ftFSfmfilenMATgbl ftFSiLVtypeVECgbl ftFSfmfcntVECgbl ftioLVtypeVECgbl
global ftFSimpnameMATgbl ftFSaggrfilenMATgbl ftFSdefuznameMATgbl
global ftFSimptypeSCAgbl ftFSaggrtypeSCAgbl ftFSdeftypeSCAgbl
global ftFSsum_lingVECgbl
global ftFSallaltMATgbl ftFSaltidxVECgbl ftFSactidxVECgbl ftFSscoresMATgbl

%
% Map the input values from the universe of discourse to the base set
%
for(i=1:ftFSNo_altCNTgbl)
        x(i,:) = ftmaptob(x(i,:),1);
end; % for loop

newqualcrit=[];
for alt = 1:ftFSNo_altCNTgbl
%====================================================================

 for r = 1:ftFSNo_rulesCNTgbl
  %------------------------------------------------------------------
  %    Have to generate the matrix "basis"
  %    Normalize this curve
  %    Find the membership values for each criteria in the rules
  %------------------------------------------------------------------
  ftFSstate_ = 1;
  while(ftFSstate_<=ftFSNo_inCNTgbl)
    if(ftFSaincMATgbl(r,ftFSstate_) ~= 0)
        %------------------------------------------------------------
        % APPLY
        % FUZZY MEMBERSHIP FUNCTION: users choice - in the string ftFSfmfSTRgbl
        %------------------------------------------------------------
        ftFSloc_ = ftFSainlMATgbl(r,ftFSstate_)+ftFSsum_lingVECgbl(ftFSaincMATgbl(r,ftFSstate_));
        cnt = ftFSfmfcntVECgbl(ftioLVtypeVECgbl(ftFSloc_));
        ftFSfmfparmMATgbl(1:cnt)= ftFSparmMATgbl(ftFSloc_,1:cnt);
        memb_val(r,ftFSstate_) = feval(ftFSfmfilenMATgbl(ftioLVtypeVECgbl(ftFSloc_),:), x(alt,ftFSaincMATgbl(r,ftFSstate_)), ftFSfmfparmMATgbl);
     %----------------------
     %----------------------
     %  normalize this value
     %----------------------
%    rl=ftFSainlMATgbl(r,ftFSstate_);
%    disp([ftFSmaxsVECgbl(rl) ftFSminsVECgbl(rl)])
     memb_val(r,ftFSstate_) = (memb_val(r,ftFSstate_)-ftFSminsVECgbl(ftFSloc_))/(ftFSmaxsVECgbl(ftFSloc_)-ftFSminsVECgbl(ftFSloc_));
    end %if
    ftFSstate_ = ftFSstate_ + 1;
    end % while loop
 end % for r loop
%=========================================================================
%-------------------------------------------------------------------------
%   Apply the ftFSrule_ (combination of criteria, linguistic variable , AND|OR)
%-------------------------------------------------------------------------
 for r = 1:ftFSNo_rulesCNTgbl
   ftFSstate_ = 1;
   while(ftFSstate_<=(ftFSNo_inCNTgbl-1))
    if(ftFSaincMATgbl(r,ftFSstate_) ~= 0)
     if(ftFSainrMATgbl(r,ftFSstate_) == 1)
       memb_val(r,1) = min(memb_val(r,1),memb_val(r,(ftFSstate_+1)));
     elseif(ftFSainrMATgbl(r,ftFSstate_) == 2)
       memb_val(r,1) = max(memb_val(r,1),memb_val(r,(ftFSstate_+1)));
     end % if
    end % if
    ftFSstate_ = ftFSstate_ + 1;
   end % while
 end % for r loop
%=========================================================================
%------------------------
% One value for each ftFSrule_
%------------------------
 ftFSrule_ = memb_val(:,1);

%=========================================================================
%-------------------------
% For each ftFSaction_ criteria
%--------------------------
 for act=1:ftFSNo_outCNTgbl
   %----------------------------------------------
   % Set up ftFSrbasisMATgbl according to the type of ftFSaction_
   %----------------------------------------------
   ftFSrbasisMATgbl(1:ftFSNo_rulesCNTgbl,:) = ftFSbasisMATgbl(ftFSaoutlMATgbl(1:ftFSNo_rulesCNTgbl,act) + ftFSNo_ilingCNTgbl,:);

   %--------------------------------------------------------------------------
   % FUZZY IMPLICATION: choosen implication function - in the string ftFSimplicateSTRgbl
   %--------------------------------------------------------------------------
   imply = feval(ftFSimpnameMATgbl(ftFSimptypeSCAgbl,:),ftFSrule_, ftFSrbasisMATgbl, ftFSNo_rulesCNTgbl, ftFSNo_stepsCNTgbl);

   %---------------------------------------------------------
   %  FUZZY AGGREGATION: use the choosen aggregation function
   %---------------------------------------------------------
   if(ftFSaggrtypeSCAgbl==15)  
                aggr = ftfsgenm(imply, ftFSNo_rulesCNTgbl, ftFSaggrparmMATgbl(ftFSaggrtypeSCAgbl,:));      
   else 
                if(ftFSNo_rulesCNTgbl==1)
                        aggr = imply;
                else
                        aggr=imply(1,:);
                        for r=2:ftFSNo_rulesCNTgbl
                                aggr(1:ftFSNo_stepsCNTgbl)=feval(ftFSaggrfilenMATgbl(ftFSaggrtypeSCAgbl,:),aggr(1:ftFSNo_stepsCNTgbl),imply(r,1:ftFSNo_stepsCNTgbl),ftFSaggrparmMATgbl(ftFSaggrtypeSCAgbl,:));
                        end % for r loop
                end % if
   end % if
   %--------------------------------------
   %  Normalize the Aggregated Fuzzy Curve
   %-------------------------------------- 
   norms  = ftfsnorm(aggr);

   %---------------------------------------------------------------
   %  DEFUZZIFIER. (defuzzify the Normalized Fuzzy Aggregate Curve)
   %                            use the choosen defuzzifier
   %---------------------------------------------------------------
   ftFSallaltMATgbl(alt,act)  = feval(ftFSdefuznameMATgbl(ftFSdeftypeSCAgbl,:),norms, ftFSbase_setVECgbl, ftFSNo_stepsCNTgbl, ftFSdefparmMATgbl(ftFSdeftypeSCAgbl,1), ftFSdefparmMATgbl(ftFSdeftypeSCAgbl,2));

 end % for act loop                        
 newqualcrit=[newqualcrit;ftFSqual_critMATgbl];
end % for alt loop 

%---------------------------------------------------------
% ftFSallaltMATgbl matrix has the rating of the alternatives as 
% ftFSallaltMATgbl(1:No of alternatives, 1: No of actions)
% The next step is to identify which of these alternatives
% qualify, and then rank those who qualify
% and provide their score in the ftFSscoresMATgbl vector and
%             the ranking in the ftFSaltidxVECgbl and ftFSactidxVECgbl vectors
% ftFSscoresMATgbl are in descending order
%---------------------------------------------------------
        
[sortallalt, ftFSaltidxVECgbl] = sort(ftFSallaltMATgbl);
ftFSscoresMATgbl=sortallalt(find(sortallalt>=newqualcrit));
ftFSactidxVECgbl=ftFSaltidxVECgbl(find(sortallalt>=newqualcrit));
%  ========================================================================   
%  CAUTION   
%  ========================================================================   
%  EXAMPLE   
   
%  ========================================================================   
%  SEE ALSO   
%   
%       FlexTools(GA)   
%  ========================================================================   
%  REFERENCE   
%    
%  Guidelines for Evolutionary Fuzzy Modeling
%  FIG LLC, Tuscaloosa, AL 35486-1477, USA, FIGLLC@AOL.COM
%
%  K. Krishnakumar, Antony Satyadas   
%  Discovering Multiple Fuzzy Models using MFMCN Algorithm   
%  ACM Computing Week Symposium of Applied Computing (SAC) Conference,  
%  Nashville, TN, March 1995.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END ftfsfdms.m     
% ftfsfdms                           ftfsfdms.m                  ftfsfdms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

