%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% FTrunenm                         FTrunenm.m                       FTrunenm
% =========================================================================  
%                          FlexTools(ENM) for Matlab                          
%                             Copyright (c) 1996                               
%                       Flexible Intelligence Group (FIG)       	             
%                   P. O. Box 861477, Tuscaloosa, AL 35486-0013, USA.            
%                                 06/05/1996                                 
% =========================================================================  
% =========================================================================   
%  PURPOSE   
%         run FlexTools(GA) GA algorithm.   
%         THIS IS A SCRIPT FILE
% .........................................................................   
%  GIVEN   
%	if restart, file named 'ga_warm.mat' which has all the ga parameters  
%           else all parameter values in appropriate variables 
% ...........................................................................   
%  OPERATION   
%         Assign values to variables from gap.dat data file   
%	  Generate an initial random population initpop()   
%	  Compute the fitness (PI) of each individual in this population by   
%         applying the decoded parameters represented by each individual   
%         to the given objective function.   
%         For ftGAngenSCAgbl generations, select new offsprings (survival of the fittest,   
%         do crossover and mutation operators, evaluate the fitness of this    
%         new population by using the decoded parameters in the given   
%	  objective function.   
% ...........................................................................   
%  RETURNS   
%         Optimized parameters, Evolved fitness (average,best) for each   
%         generation in the form of a graph and file.   
% ...........................................................................   
%  APPLICATION   
%         In GA applications, called based on your choice in ga_info.   
%         Can be used for          
%			*	Classification and Learning   
%			*	Numerical function optimization   
%			*	Image processing   
%			*	Combinatorial optimization such as travelling salesman   
%				problem, bin packing, job shop scheduling   
%			*	Machine Learning   
%			*            Design   
%			*	Fuzzy Modeling   
%			*	Neural Network learning   
%			*	Time tabling   
%			*	Management science   
%			*	Game Playing   
%			*	Robotics   
%			*	Cognitive Science   
%			*	Demographics   
%			*	Finance, Economics   
%			*	Aerospace   
%			*	Prediction, Control   
%			*	Geology   
%			*	   
% ...........................................................................   
%  USE WITH   
%   
%	MUST   
%		ftga.m, ftgacold.m, ftgawarm.m, FlexTools(GA), Matlab  
%	MAY   
%		FlexTools(ENM)   
%   USES  
%       FTinit.m, FTinipop.m, FTdecode.m, FTnich.m, FTstats.m, FTgen.m,   
%       rand, if, load, figure, hold, for, feval, while, sort,   
%       subplot, plot, xlabel, ylabel, axis, title, pause, save  
%  ==========================================================================   
%  SYNTAX  /  USAGE   
%       ftrunenm   
   
%  ==========================================================================   
%  DESCRIPTION -  FLEXTOOL APPROACH   
%   
%  GAs is a probabilistic optimization algorithm. It is a guided random    
%  search technique based evolutionary algorithm (based on natural selection    
%  principles) which evolves the potential solutions over generations of    
%  reproduction using selection, crossover, and mutation techniques. It    
%  combines the Darwinian survival-of-the-fittest strategy to eliminate unfit    
%  features and use random information exchange, with exploitation of a priori    
%  knowledge (predefined and that available in old solutions)   
%   
%  GA    
%       *   work with a ftGAcodingSCAgbl of the paramter set, not the parameters themselves   
%       *   search from a population of points, not a single point   
%       *   use probabilistic, not deterministic transition rules   
%   
%  DESCRIPTION - OTHER APPROACHES   
%  Other search techniques include   
%		*	Calculus based - direct (fibonacci,newton) or indirect   
%		*	Enumerative    - (dynamic programming)   
%		*       Simulated annealing (another type of random search technique)   
%                       Deals with only one candidate solution at a time. Information   
%			from previous moves are not saved for future use)   
%		*       Brute force or enumerated random search (rarely used by itself)   
%		*	Gradient methods (hill climbing. good for unimodal functions.   
%			(tends to get stuck in local maxima or minima)   
%		*	Iterated Search (combines random and gradient search. no overall   
%			picture of the domain is available. simple. Good if the function   
%			Does not have too many local maxima or minima. does not    
%			differentiate between low and high fitness areas in terms of    
%			allocating trials).   
%		*   evolutionary strategy (another type of evolutionary algorithm)   
%   
%  DISCUSSION   
%  Here we use:   
%     *  Sequential (in contrast to parallel option)   
%     *	 Coding 	options: binary, real   
%     *  Replacement policy - Regular GA uses generational (steady ftFSstate_ GA, Micro GA   
%                             has steady ftFSstate_ replacement policy)   
%     *  Haploid    (the chromosome consists of a single sequence of genes)   
%                   (diploid - 2 seqeuences/ a pair. useful in online   
%		               adaptive modes - need to carry dominance info. increases   
%			       overhead)   
%     *  Selection  options: Roulette Wheel (proportional) selection,   
%                            Tournament selection   
%                            Ranking selection   
%     *  Crossover  options: one-point, two-point, multi-point crossovers,   
%			     with a crossover rate ftGApcrossSCAgbl   
%     *  Mutation   flip bits, with a mutation rate ftGApmutateSCAgbl   
%     *    
%  ==========================================================================   
%  VARIABLE  DICTIONARY   
%      GLOBAL   
%	  ftNNtypeSCAgbl	:  type of neural system
%         ftGAnpopSCAgbl		:  population size (Always odd) #of individuals   
%         ftGAlchromSCAgbl	:  Length of the chromosome/string   
%         ftGAnopSCAgbl		:  number of parameters   
%         ftGAmmVECgbl		:  1 for minimization, - 1 for maximization   
%         ftGApmutateSCAgbl	:  probability of mutation (mutation rate)   
%         ftGApcrossSCAgbl	:  probability of crossover (crossover rate)   
%         ftGAncrossSCAgbl	:  # of crossover points   
%         ftGAnbitsVECgbl		:  # of bits assigned for each parameter ftGAnbitsVECgbl(1:ftGAnopSCAgbl)   
%         ftGApslopeVECgbl	:  slopes for the parameters ftGAnbitsVECgbl(1:ftGAnopSCAgbl,1)   
%         ftGApcVECgbl		:  y intercept for the paremeters ftGApcVECgbl(1:ftGAnopSCAgbl,1)   
%        ftGA_choiceSCAgbl	:  1:regular ga; 2:micro-ga; 3:steady ftFSstate_ (SS) ga, 4: Flex-GA   
%        sobj		:  Number of characters in function name   
%        ftGAobjfuncVECgbl	:  name of the function is stored here   
%        ftGAnoofobjSCAgbl	:  # of objectives   
%        ftGAnichtestSCAgbl	:  1: no niching; 2: with niching   
%        ftGAnoofpeaksSCAgbl	:  # of assumed peaks for niching algorithm   
%        ftGAcodingSCAgbl		:  1: binary; 2:logarithmic; 3:real   
%        ftGAnselectSCAgbl	:  1: Tournament selection; 2: Roulette; 3: Ranking   
%        ftGAtsizeSCAgbl		:  Size of the tournament    
%        ftGAsspopSCAgbl		:  Population with constant fitnesses used in SS ga.   
%        ftGAregenSCAgbl		:  # of generations per micro-ga loop   
%        nmax		:   
%        nmin		:   
%       LOCAL   
%         i,j,k		:  iteration variables   
%         ftGApminVECgbl		:  min. values for parameters ftGApminVECgbl(1:ftGAnopSCAgbl)   
%         ftGApmaxVECgbl		:  max. values for parameters ftGApmaxVECgbl(1:ftGAnopSCAgbl)   
%         gno		:  generation counter     
%         genn		:  stores generation number genn(1:ftGAngenSCAgbl)   
%         bgno		:  generation count for display purpose   
%         maxi		:  max. PI value of each generation maxi(1:ftGAngenSCAgbl)   
%         mini		:  min. PI value of each generation mini(1:ftGAngenSCAgbl)   
%         avg		:  avg. of PI values of each generation avg(1:ftGAngenSCAgbl)   
%         sd		:  standard deviation - PI vals - each gen. sd(1:ftGAngenSCAgbl)   
%         noofeval	:  Number of evaluations noofeval(1:ftGAngenSCAgbl)   
%         oldpop	:  Old population of individuals oldpop(1:ftGAnpopSCAgbl,1:ftGAlchromSCAgbl)   
%         newpop	:  New population of individuals newpop(1:ftGAnpopSCAgbl,1:ftGAlchromSCAgbl)   
%         savePI	:  Intermediate variables   
%         nichePI	:  Intermediate variables   
%         p		:  decoded parameters p(1:ftGAnopSCAgbl)   
%         p_p		:  save p in amatrix   
%         sort_y	:   
%         sort_i	:   
%         bestp		:  best parameters    bestp(1:ftGAnopSCAgbl)   
%         savepop	:  save best individual (nmin) savepop(1,1:ftGAlchromSCAgbl)   
%         genp		:  variables for plotting   
%         nfnp		:          ,,   
%         minp		:          ,,   
%         maxp		:          ,,   
%         avgp		:          ,,   
%  ==========================================================================   
%  ALGORITHM   
%	10	create a population of individuals. each bunch of bits within   
%		each individual is mapped to the paramter value we are trying to   
%		optimize.   
%	20      evaluate each individual (decode the parameters and apply them   
%		to the objective function (fitness function) and obtain the fitness   
%		of each individual.   
%	30	select the best based on some strategy   
%	40	apply reproduction strategies such as crossover and mutation   
%	50	obtain a new generation of population of better individuals   
%	60      repeat 10-50 for ftGAngenSCAgbl generations   
%	70	draw graphs, compute statistics. could be done within 10-50   
%	80	Repeat 10-70 seq times (if sequencer is used in EA) else use NN learning
%		Default is 1.
%	90	Repeat 10-80 cyc times (if cycler is being used). Default = 1.
%  ==========================================================================   
%  CODE    
   
% GLOBAL   

function ftrunenm

global ftGApslopeVECgbl ftGApcVECgbl bestp bestp_n nmin nmax bestP nichPI
global ftGAgapVECgbl ftGAfunpVECgbl ftGAch_flagSCAgbl ftGAcodingSCAgbl sobj 
global ftGAobjfuncVECgbl ftGAnoofobjSCAgbl ftGAmmVECgbl ftGAnopSCAgbl   
global ftGApmaxVECgbl ftGApminVECgbl ftGApresVECgbl ftGAnbitsVECgbl ftGAlchromSCAgbl initfunc   
global ftGAnichtestSCAgbl ftGAnoofpeaksSCAgbl ftGAnselectSCAgbl 
global ftGAtsizeSCAgbl ftGAnpopSCAgbl ftGAsspopSCAgbl ftGAngenSCAgbl ftGAregenSCAgbl   
global ftGApcrossSCAgbl ftGApmutateSCAgbl ftGAncrossSCAgbl ftGAp1SCAgbl ftGAp2SCAgbl   

global fts_tableMATgbl ftm_tableMATgbl ftGAp_typeVECgbl
global ftGApiVECgbl ftGApVECgbl ftGAmchildSCAgbl
global two_power_k  ftGApresuVECgbl
global ftGA_choiceSCAgbl  ftGAch_flagSCAgbl
global ftGAminiVECgbl ftGAmaxiVECgbl ftGAavgVECgbl ftGAgennVECgbl   
global ftGAnooffevalVECgbl ftGAbestpMATgbl 

% NN global declarations

global ftNNpp_typeMATgbl
global ftscr_cntSCAgbl ftflag_edtGUI
global ftNNtitleVECgbl ftNNtypeSCAgbl
global ftNNspecialSCAgbl ftNNpp_specMATgbl
global ftNNfn_errMATgbl ftNNer_typeSCAgbl ftNNpp_ertypeMATgbl
global ftNNer_betaSCAgbl
global ftNNlearnSCAgbl ftNNpp_lrnMATgbl
global ftNNconvSCAgbl ftNNpp_cnvMATgbl ftNNpp_actMATgbl
global ftNNno_inSCAgbl ftNNno_hidSCAgbl ftNNno_outSCAgbl ftNNno_totSCAgbl
global ftNNh_layerSCAgbl ftNNpp_hlayerMATgbl ftNNno_hidVECgbl
global ftNNlrSCAgbl ftNNmrSCAgbl
global ftNNn_epSCAgbl ftNNe_thSCAgbl
global ftNNv_pcntSCAgbl ftNNasetFLGgbl
global ftNNassocMATgbl    ftNNassnCNTgbl   ftNNassocOPTgbl
global ftNNweightMATgbl  ftNNweightOPTgbl 
global ftNNwrnd_pcntSCAgbl ftNNfn_actMATgbl

global ftNNn_inameVECgbl ftNNn_imaxVECgbl ftNNn_iminVECgbl ftNNn_itypeVECgbl
global ftNNn_hnameVECgbl ftNNn_hmaxVECgbl ftNNn_hminVECgbl ftNNn_htypeVECgbl
global ftNNn_onameVECgbl ftNNn_omaxVECgbl ftNNn_ominVECgbl ftNNn_otypeVECgbl

global ftNNbase_maxSCAgbl ftNNbase_minSCAgbl ftNNbase_resSCAgbl

global ftNNn_iparmVECgbl ftNNn_hparmVECgbl ftNNn_oparmVECgbl
global ftNNtrainFLGgbl ftNNsimFLGgbl ftNNst_flagSCAgbl
global ftNNv_stringMATgbl ftNNv_countSCAgbl  ftNNpl_freqSCAgbl

global nn_in nn_desired nn_out

% ENM global declarations

global ftENMlrnOPTgbl     ftENMlrnNMATgbl    ftENMmaxSeqCNTgbl
global ftENMSeqChVECgbl   ftENMSeqChNMATgbl  ftENMeaConVECgbl
global ftENMSeqTimeVECgbl ftENMeaWtVECgbl    ftENMeaOptNMATgbl 
global ftENMselInVECgbl   ftENMynNameMATgbl  ftENMassn1MATgbl 
global ftENMassn1CNTgbl   ftENMassn2MATgbl   ftENMassn2CNTgbl
global ftENMassn3MATgbl   ftENMassn3CNTgbl   ftENMassn4MATgbl
global ftENMassn4CNTgbl   ftENMwts1MATgbl    ftENMwts1CNTgbl 
global ftENMwts2MATgbl    ftENMwts2CNTgbl    ftENMwts3MATgbl
global ftENMwts3CNTgbl    ftENMwts4MATgbl    ftENMwts4CNTgbl
global ftENMsameVECgbl    ftENMSeqCNTgbl     ftENMnn_inMATgbl
global ftENMFLGgbl        ftENM2FLGgbl
global ftNNfnflagSCAgbl   ftnndef_flag
global ftENMltypeFLGgbl   ftENMlchromrgbl    lchromr
global ftENMrwtlIDXgbl    ftENMcwtlIDXgbl    ftENMrassnlIDXgbl   
global ftENMcassnlIDXgbl  ftENMrestartFLGgbl
%global ftENMCycCNTgbl     ftENMCycfreqSCAgbl

global ftXYscaleVECgbl
if ftENMrestartFLGgbl
   load ftenwarm
   ftENMSeqCNTgbl=1;
   ftENMrestartFLGgbl=1;
end

ftENMnn_inMATgbl=nn_in;
ftNNpl_freqSCAgbl=5;

 % Uses the sequencer
 for(seq=1:ftENMSeqCNTgbl)

  % if learning choice is EA 
  if(ftENMSeqChVECgbl(1,seq)==3)
    ftenmin(seq);

    % Obtain number of generations from the sequencer!
    if ftENMrestartFLGgbl ==0, 
	ftGAngenSCAgbl = ftENMSeqTimeVECgbl(seq);
    end

    ftNNpl_freqSCAgbl=999999;

    ftNNtrainFLGgbl = 0;
    ftNNsimFLGgbl   = 1;
  %=================== 
  % Obj Function parameters 
  %===================
  i=1;	         sobj	= ftGAfunpVECgbl(i);          
                    	objfunc	= setstr(ftGAfunpVECgbl((i+1):(i+sobj)));      
  i=i+sobj+1;           siobj	= ftGAfunpVECgbl(i);      
                    	initfunc= setstr(ftGAfunpVECgbl((i+1):(i+siobj)));      
  i=i+siobj+1;  	Noofobj	= ftGAfunpVECgbl(i);       
                    	mm	= ftGAfunpVECgbl((i+1):(i+Noofobj));      
  i=i+Noofobj+1;        nop	= ftGAfunpVECgbl(i);      
  i=i+1;             	coding	= ftGAfunpVECgbl(i);     
			ftGAcodingSCAgbl=coding;   
                    	pmax	= ftGAfunpVECgbl((i+1):(i+nop));      
  i=i+nop;           	pmin	= ftGAfunpVECgbl((i+1):(i+nop));      
  i=i+nop;           	pres	= ftGAfunpVECgbl((i+1):(i+nop));      
  i=i+nop;           	nbits	= ftGAfunpVECgbl((i+1):(i+nop));      
  i=i+nop+1;       	lchrom	= ftGAfunpVECgbl(i);

  if size(ftGAfunpVECgbl) > i
  	            	presu	= ftGAfunpVECgbl((i+1):(i+nop)); 
  else
	    		presu=ftGApresuVECgbl;
  end
  if nargin>2
    if isempty(ifunc)==0, initfunc=ifunc; end
  end
  if nargin>1
    if isempty(ofunc)==0, objfunc=ofunc; end
  end
  %==============
  % GA parameters
  %==============
  ftGAch_flagSCAgbl=0;
  i=1; 	nichtest=ftGAgapVECgbl(i);
  i=2; 	Noofpeaks=ftGAgapVECgbl(i);
  i=3; 	nselect=ftGAgapVECgbl(i);
  i=4; 	Tsize=ftGAgapVECgbl(i);
  i=5; 	npop=ftGAgapVECgbl(i);
  i=6; 	sspop=ftGAgapVECgbl(i);
  npop=ftGAnpopSCAgbl;
  sspop=ftGAsspopSCAgbl;
  if sspop >= npop, sspop=round(npop/2); end
  i=6; 	ftGAgapVECgbl(i)=sspop;
  if rem(npop,2)==0, npop=npop+1;    i=5; ftGAgapVECgbl(i)=npop; end
  if rem(sspop,2)==0, sspop=sspop+1; i=6; ftGAgapVECgbl(i)=sspop; end
  ftGAnpopSCAgbl=npop;
  ftGAsspopSCAgbl=sspop;
  i=7; 	ngen=ftGAgapVECgbl(i);
  i=8; 	regen=ftGAgapVECgbl(i);
  i=9; 	pcross=ftGAgapVECgbl(i);
  i=10;	pmutate=ftGAgapVECgbl(i);
  i=11;	ncross=ftGAgapVECgbl(i);
  i=12;	p1=ftGAgapVECgbl(i);
  i=13;	p2=ftGAgapVECgbl(i);

  lchromr = ftENMlchromrgbl;

  gap    = ftGAgapVECgbl;
  pc     = ftGApcVECgbl;
  pslope = ftGApslopeVECgbl;

    %----------------------------          
    % Initialize Fitness Function
    %----------------------------   
    if initfunc~='x'   
       feval(initfunc);   
    end   
    %--------   
    % figure   
    %--------
    GA_stop_flag=0;
    FigurePos_GAplot=[70    25   490   350].*ftXYscaleVECgbl;
    ftFSlala_=['FlexTool(ENM)--GA Evolution. Stop:<ctrl-C>. Seq#:' num2str(seq) '.'];
    conn_ = ftENMeaOptNMATgbl(ftENMeaConVECgbl(1,seq),:);
    wts_  = ftENMeaOptNMATgbl(ftENMeaWtVECgbl(1,seq),:);
    if(ftENMselInVECgbl(1,seq)==1)
	    ins_  =  'No';
    else
	    ins_  =  'Yes';
    end;
    ftFSlala_ = [ftFSlala_ ' Conn:' conn_ ', Wts:' wts_ ', In:' ins_ '.'];
    fig_ftga=ftfigure('fig_ftga',ftFSlala_,FigurePos_GAplot,'on','figure');
    ftFSpp2_=['FlexTool(ENM) will display Evolution plots after two Generations. '];
    ftFSpp3_=[0.1 0.48 0.8 0.085];
    ftscrtxt(fig_ftga,ftFSpp2_,ftFSpp3_);
    clear ftFSpp2_ ftFSpp3_;

    %------------          
    % If RESTART
    %------------   
    if ftENMrestartFLGgbl
       disp(['yes'])
       ftGAngenSCAgbl=ftGAngenSCAgbl+gno;
    else  
       %------------
       % FRESH START
       %-------------------------------
       % clear old variables if existed
       %-------------------------------
       if exist('oldpop') 
          clear  oldpop newpop p_p savepop PI savePI nichPI maxi mini avg sd 
          global oldpop newpop p_p savepop PI savePI nichPI maxi mini avg sd 
	  ftGAmaxiVECgbl=0;
	  ftGAminiVECgbl=0;
	  ftGAavgVECgbl =0;
       end; %if
       if exist('scaled_P')
          clear  scaled_P decide mydist myoutpair 
          global scaled_P decide mydist myoutpair
       end; %if
       if exist('min_PI')
          clear min_PI max_PI avg_PI
          global min_PI max_PI avg_PI
       end; %if
       %---------------------------------   
       % Set up ftGApminVECgbl, ftGApmaxVECgbl, ftGApresVECgbl, ftGAnopSCAgbl
       %---------------------------------   
       outs=ftenmin(seq);
       if outs==0, return; end
       %-------------------------------  
       % Random PI Initialization   
       %-------------------------------  
       PI=rand(ftGAnpopSCAgbl,ftGAnoofobjSCAgbl);           
       ftGAmchildSCAgbl=1; 	%No. of asexual children for stochastic ftGAcodingSCAgbl	
       gno=1;   
       ftGAgennVECgbl(gno)=gno;   
       ftGAnooffevalVECgbl(gno)=((gno-1)*(ftGAnpopSCAgbl-ftGAsspopSCAgbl)*ftGAmchildSCAgbl+ftGAnpopSCAgbl);    
       %--------------------------          
       % Initialize Population   
       %--------------------------   
       %--------------------- 
       % CALL FTinipop.m 
       %---------------------  
       oldpop=ftinipop(ftGAnpopSCAgbl,ftGAlchromSCAgbl);   

       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       % If using Cycler       
       %if (cyc>1)
       %   [popn_,parn_]=size(oldpop);
       %   howmany_ = fix((ftGAnpopSCAgbl*ftENMCycfreqSCAgbl/100)*(cyc/ftENMCycCNTgbl));
       %   oldpop(1:howmany_,1:parn_)=bestP(1:parn_);
       %end; % if
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       ftENMrestartFLGgbl=0;   
  
       %--------------------------------   
       % decode parameters   
       % compute PI by calling the   
       % objective function   
       %--------------------------------   
       tic;
       for ii=1:ftGAnpopSCAgbl   
          %---------------------  
          % CALL FTdecode.m  
          %---------------------  
          p=ftdecode(oldpop(ii,:),ftGAcodingSCAgbl,ftGAnopSCAgbl,ftGApresVECgbl,presu,ftGApmaxVECgbl,ftGAnbitsVECgbl,ftGApcVECgbl,ftGApslopeVECgbl,two_power_k);
	  if ftGA_choiceSCAgbl==4 %Flex_GA
             p_p(:,ii)=ftGApVECgbl';
             PI(ii,:)=ftGApiVECgbl;
          else   
             p_p(:,ii)=p';   
             PI(ii,:)=feval(ftGAobjfuncVECgbl,p);
             PI(ii,:)=PI(ii,:).*ftGAmmVECgbl;
          end
          ftstatus(ii,gno,ftGAnpopSCAgbl,ftGAngenSCAgbl,ii); 
        end % ii for loop   
        savePI=PI;   
        saveP_P=p_p;
        %---------------------------------   
        % CALL FTnich.m (do niching)  
        %---------------------------------   
        if ftGAnichtestSCAgbl == 2   
           nichPI=ftnich(ftGAmmVECgbl, ftGAnpopSCAgbl,ftGAnopSCAgbl,PI(:,1),p_p,ftGApmaxVECgbl,ftGApminVECgbl,ftGAnoofpeaksSCAgbl);    
        else   
           nichPI=PI;   
        end % if   
    end; %if not restart

    %-------------------------------------------   
    % CALL FTstats.m (compute statistics)  
    %-------------------------------------------   
    [ftGAmaxiVECgbl(gno),ftGAminiVECgbl(gno),ftGAavgVECgbl(gno),sd(gno),nmax,nmin]=ftstats(PI(:,1));   
    %-------------------------   
    % For each generation    
    %-------------------------   
    %....................................................................   
    % Obtain number of generations from the sequencer!
    ftGAngenSCAgbl = ftENMSeqTimeVECgbl(seq);

    while (gno < ftGAngenSCAgbl)
       %--------------------   
       % ga or micro-ga loop   
       %--------------------   
       % -------------------------------------------------------   
       for k=1:ftGAregenSCAgbl
          if ftGA_choiceSCAgbl==4, ftGAmchildSCAgbl=2; end	%No. of asexual children for stochastic ftGAcodingSCAgbl	
          if GA_stop_flag==1, clf; close; return; end   
          time_per_gen=toc;
          tic;
          [sort_y,sort_i]=sort(savePI);   
          %-------------------------------------------------   
          % CALL FTgen.m (generate new population)   
          %-------------------------------------------------   
          newpop=ftgen(oldpop,nichPI,ftGAgapVECgbl,ftGAcodingSCAgbl,ftGAnopSCAgbl);   
          for i=1:ftGAsspopSCAgbl   
             newpop((ftGAnpopSCAgbl-ftGAsspopSCAgbl+i),:)=oldpop(sort_i(i),:);   
             PI((ftGAnpopSCAgbl-ftGAsspopSCAgbl+i),:)=savePI(sort_i(i),:);   
             p_p(:,(ftGAnpopSCAgbl-ftGAsspopSCAgbl+i))=saveP_P(:,sort_i(i));   
          end % i for loop   
          %-----------------   
          % increment gno   
          %------------------   
          gno=gno+1;    
  
          for j=1:(ftGAnpopSCAgbl-ftGAsspopSCAgbl)   
              %---------------------  
              % CALL FTdecode.m  
              %---------------------  
              p=ftdecode(newpop(j,:),ftGAcodingSCAgbl,ftGAnopSCAgbl,ftGApresVECgbl,presu,ftGApmaxVECgbl,ftGAnbitsVECgbl,ftGApcVECgbl,ftGApslopeVECgbl,two_power_k);
              if ftGA_choiceSCAgbl==4 %Flex_GA
                 p_p(:,j)=ftGApVECgbl';
                 PI(j,:)=ftGApiVECgbl;
              else   
                 p_p(:,j)=p';   
                 %-------------------------------------------------------  
                 % CALL objfunc.m or other objective function  
                 %-------------------------------------------------------  
                 PI(j,:)=feval(ftGAobjfuncVECgbl,p);
                 PI(j,:)=PI(j,:).*ftGAmmVECgbl;
              end
              ftstatus(j,gno,ftGAnpopSCAgbl,ftGAngenSCAgbl,j); 
          end % j for loop   
          savePI=PI;
          saveP_P=p_p;
          %---------------------------------   
          % CALL FTnich.m (do niching)  
          %---------------------------------   
          if ftGAnichtestSCAgbl == 2   
             nichPI=ftnich(ftGAmmVECgbl,ftGAnpopSCAgbl,ftGAnopSCAgbl,PI(:,1),p_p,ftGApmaxVECgbl,ftGApminVECgbl,ftGAnoofpeaksSCAgbl);    
          else   
             nichPI=PI;   
          end % if   
   
          ftGAgennVECgbl(gno)=gno;   
          ftGAnooffevalVECgbl(gno)=((gno-1)*(ftGAnpopSCAgbl-ftGAsspopSCAgbl)*ftGAmchildSCAgbl+ftGAnpopSCAgbl);
          oldpop=newpop;   
          %------------------------------------------   
          % CALL FTstats.m (compute statistics)   
          %------------------------------------------   
          [ftGAmaxiVECgbl(gno),ftGAminiVECgbl(gno),ftGAavgVECgbl(gno),sd(gno),nmax,nmin]=ftstats(PI(:,1));    
          %------------------------   
          % get best parameters   
          %------------------------   
          %------------------------  
          % Store bestp vector  
          %------------------------  
          ftGAbestpMATgbl = p_p(:,nmin)';   
          savepop(1,:)=oldpop(nmin,:);   
      
          if gno > 5, bgno=gno-5; else bgno=1; end   
          genp=ftGAgennVECgbl(bgno:gno);   
          nfnp=ftGAnooffevalVECgbl(bgno:gno);   
          minp=ftGAminiVECgbl(bgno:gno);   
          maxp=ftGAmaxiVECgbl(bgno:gno);   
          avgp=ftGAavgVECgbl(bgno:gno);   
          %--------------------  
          % PLOT GRAPH  
          %--------------------
          ftFSpp2_=['Every generation will take approximately ',...
                     num2str(fix(time_per_gen)) ' Seconds. '];
          ftFSpp3_=[0.1 0.48 0.8 0.085];
          ftscrtxt(fig_ftga,ftFSpp2_,ftFSpp3_);
  
          subplot(3,3,7)
          plot(genp,(maxp*ftGAmmVECgbl(1)),'r', genp,(minp*ftGAmmVECgbl(1)),'g',genp,(avgp*ftGAmmVECgbl(1)),'w');    
          xlabel('# of Gens');   
          ylabel('Fitness Value');
  
          ftstatus(ftGAnpopSCAgbl,gno,ftGAnpopSCAgbl,ftGAngenSCAgbl,0); 
          subplot(3,3,8);
          plot(p_p(ftGAp1SCAgbl,:),p_p(ftGAp2SCAgbl,:),'+');   
  
          axis([ftGApminVECgbl(ftGAp1SCAgbl) ftGApmaxVECgbl(ftGAp1SCAgbl)     ftGApminVECgbl(ftGAp2SCAgbl) ftGApmaxVECgbl(ftGAp2SCAgbl)]);
          xlabel('x=P1; y=P2');   
   
          subplot(3,1,1)
          plot(nfnp,(minp*ftGAmmVECgbl(1)),'g'); 
          title('white = average, green = best, red = worst');   
          xlabel('Number of Function Evaluations');   
          ylabel('Fitness Value');
          drawnow
          %---------  
          % SAVE  
          %---------  
	  bestp = ftGAbestpMATgbl;
	  mini  = ftGAminiVECgbl;
          maxi  = ftGAmaxiVECgbl;
          avg   = ftGAavgVECgbl;
          nooffeval=ftGAnooffevalVECgbl;	

          ftenmeld(ftGAbestpMATgbl(1,:));
          bestP=ftGAbestpMATgbl;
          save ftenwarm.mat;   
      end % k for loop (ftGAregenSCAgbl)
      %------------------------------------------------------------   
      %----------   
      % Micro GA    
      %----------
      if ftGA_choiceSCAgbl == 2    
          %---------------------
          % CALL FTinipop.m
          %---------------------
          oldpop=ftinipop(ftGAnpopSCAgbl,ftGAlchromSCAgbl);   
          oldpop(ftGAnpopSCAgbl,:)=savepop(1,:);   
          for j=1:ftGAnpopSCAgbl   
             %---------------------  
             % CALL FTdecode.m  
             %---------------------  
	     p=ftdecode(oldpop(j,:),ftGAcodingSCAgbl,ftGAnopSCAgbl,ftGApresVECgbl,presu,ftGApmaxVECgbl,ftGAnbitsVECgbl,ftGApcVECgbl,ftGApslopeVECgbl,two_power_k);
             if ftGA_choiceSCAgbl==4 %Flex_GA
                p_p(:,j)=ftGApVECgbl';
                PI(j,:)=ftGApiVECgbl;
             else   
                p_p(:,j)=p';   
                %-------------------------------------------------------  
                % CALL objfunc.m or other objective function  
                %-------------------------------------------------------  
                PI(j,:)=feval(ftGAobjfuncVECgbl,p);
                PI(j,:)=PI(j,:).*ftGAmmVECgbl;
             end;
             ftstatus(j,gno,ftGAnpopSCAgbl,ftGAngenSCAgbl,j); 
          end % j for loop (ftGAnpopSCAgbl) 
          savePI=PI;
          saveP_P=p_p;
          nichPI=PI;
          %------------------------------------------   
          % CALL FTstats.m (compute statistics)   
          %------------------------------------------   
          [ftGAmaxiVECgbl(gno),ftGAminiVECgbl(gno),ftGAavgVECgbl(gno),sd(gno),nmax,nmin]=ftstats(PI);   
      end % if   

      %-------------------------------------------------------  
      % Perform clustering at the end if Niching was requested  
      %-------------------------------------------------------  

      if ftGAnichtestSCAgbl == 2 & gno==ftGAngenSCAgbl
         global decide
         for i=1:ftGAnpopSCAgbl    
            scaled_P(:,i) = p_p(:,i) ./ ((ftGApmaxVECgbl-ftGApminVECgbl)');   
         end % i for loop 
         [decide,mydist,myoutpair]=ftcclust(scaled_P',ftGAnoofpeaksSCAgbl,2,5);
         for j=1:ftGAnoofpeaksSCAgbl
            ic=0;
            ijk_np=j; 
            if ijk_np > size(decide,1), ijk_np=size(decide,1); end 
            for i=1:ftGAnpopSCAgbl
               if decide(ijk_np,i)~= 0, ic=ic+1; adum(ic)=decide(ijk_np,i); end
            end
            [yy,ii]=min(PI(adum(:)));
            bestp_n(j,:)=p_p(:,adum(ii))';
            clear adum
         end % end ftGAnoofpeaksSCAgbl loop
         clear yy ii ic 
      end % ftGAnichtestSCAgbl loop

      bestp = ftGAbestpMATgbl;
      mini  = ftGAminiVECgbl;
      maxi  = ftGAmaxiVECgbl;
      avg   = ftGAavgVECgbl;
      nooffeval=ftGAnooffevalVECgbl;
      genn  = ftGAgennVECgbl;

     ftenmeld(bestP(1,:));
     save ftenwarm.mat;
    end % ftGAngenSCAgbl while loop   

    %.............................................................   
    %----------------   
    % modify values   
    % based on   
    % max. or min.   
    %----------------   
    min_PI=ftGAminiVECgbl*ftGAmmVECgbl(1);   
    max_PI=ftGAmaxiVECgbl*ftGAmmVECgbl(1);   
    avg_PI=ftGAavgVECgbl*ftGAmmVECgbl(1);

    for i=1:ftGAnpopSCAgbl
        PI(i,:)=PI(i,:).*ftGAmmVECgbl;
    end
    %---------  
    % SAVE  
    %---------  
    if ftGAnichtestSCAgbl==2
       bestP=bestp_n;
       ftGAbestpMATgbl=bestP;
    else
       bestP=ftGAbestpMATgbl;   
    end
    ftenmeld(bestP(1,:));
    save ftenwarm.mat;   
  
  % Learning option is NN
  elseif(ftENMSeqChVECgbl(1,seq)==2)

    % Obtain number of generations from the sequencer!
    ftGAngenSCAgbl = ftENMSeqTimeVECgbl(seq);

    ftNNtrainFLGgbl = 1;
    ftNNsimFLGgbl   = 0;
    ftNNn_epSCAgbl = ftENMSeqTimeVECgbl(seq);
    % call nn learning here
    nn_out=ftnntrn(nn_in,nn_desired)
    disp(['FlexTool(ENM): Finished']);
  end; %if

end; % for seq

%end; % for cyc

%  ==========================================================================   
%  CAUTION         
%  		* make sure gap.dat is available   
%		* ensure that appropriate GA parameters have been chosen   
%  ==========================================================================   
%  EXAMPLE   
%  ==========================================================================   
%  SEE ALSO   
%   
%	FlexTools(ENM)   
%  ==========================================================================   
%  REFERENCE   
%Holland, J. H.,   
%Adaptation in Natural andArtificial Systems,   
%Ann Arbor, MI, University of Michigan Press, 1975.   
%   
%Koza, John, K.,   
%Genetic Programming,   
%MA, MIT Press, 1992.   
%   
%David Goldberg,   
%Genetic Algorithms in Search, Optimization, and Machine Learning,   
%MA, Addison-Wesley, 1989.   
%   
%Jorg Heitkotter and David Beasley,   
%The Hitch-Hiker's guide to Evolutionary Computation,   
%FAQ in comp.ai.genetic usenet newsgroup, available via anonymous ftp frm    
%rtfm.mit.edu:pub/usenet/news.answers/ai-faq/genetic/, 1994.   
%   
%Antony Satyadas and K. Krishnakumar,   
%Evolutionary Fuzzy Techniques for Fuzzy Controller Synthesis,   
%Proc. of the First Industry/University Symposium on Research for    
%Future Supersonic and Hypersonic Vehicles, NASA IEEE, TSI Press,    
%New Mexico, 148--155, December 1994.   
%   
%K. Krishnakumar,   
%Genetic Algorithms: Theory and Control Applications,   
%Tutorial, Scotland, UK, 1994.   
%   
%David Beasley, David R. Bull, and Ralph R. Martin,   
%An Overview of Genetic Algorithms: Part 1, Fundamentals,   
%University Computing, Vol. 15, No. 2, pp. 58-69, 1993.   
%   
%David Beasley, David R. Bull, and Ralph R. Martin,   
%An Overview of Genetic Algorithms: Part 2, Research Topics,   
%University Computing, Vol. 15, No. 4, pp. 170-181, 1993.   
%   
%M. Srinivas and Lalit M. Patnaik,   
%Genetic Algorithms: A Survey,   
%Computer, Vol. 27, No. 6, pp. 17-26, June 1994.   
%   
%Jose L. Ribeiro Filho, Philip C. Treleaven, and Cesare Alippi   
%Genetic-Algorithm Programming Environments   
%Computer, Vol. 27, No. 6, pp. 28-43, June 1994.   
%  ==========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END FTrunenm.m   
% FTrunenm                         FTrunenm.m                       FTrunenm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
